﻿#include <iostream>
#include <vector>
#include <sstream>
#include <string>
#include "histogram.h"
#include "svg.h"
#include "text.h"
#include <curl/curl.h>

using namespace std;

struct Input {
    vector<double> numbers;
    size_t bin_count{};
};

Input input_data(istream& in, bool prompt) {
    if (prompt) cerr << "Enter number count: ";
    size_t number_count;
    in >> number_count;
    Input data;
    data.numbers.resize(number_count);
    if (prompt) cerr << "Enter " << number_count << " numbers: ";
    for (size_t i = 0; i < number_count; i++) {
        in >> data.numbers[i];
    }
    if (prompt) cerr << "Enter bin count: ";
    in >> data.bin_count;
    return data;
}

static size_t write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
    size_t data_size = item_size * item_count;
    auto* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}

Input download(const string& address) {
    stringstream buffer;
    CURL* curl = curl_easy_init();

    curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
    curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
    CURLcode res = curl_easy_perform(curl);
    if (res != CURLE_OK) {
        cerr << "curl_easy_perform() failed: " << curl_easy_strerror(res) << "\n";
        curl_easy_cleanup(curl);
        exit(1);
    }

    double download_speed = 0.0;
    curl_easy_getinfo(curl, CURLINFO_SPEED_DOWNLOAD, &download_speed);
    cerr << "Download speed: " << download_speed << " bytes/sec\n";

    curl_easy_cleanup(curl);
    return input_data(buffer, false);
}

int main(int argc, char* argv[]) {
    curl_global_init(CURL_GLOBAL_ALL);
    Input in;
    if (argc > 1) {
        in = download(argv[1]);
    }
    else {
        in = input_data(cin, true);
    }
    const auto bins = make_histogram(in.numbers, in.bin_count);
    show_histogram_svg(bins);
    show_histogram_text(bins);
    curl_global_cleanup();
    return 0;
}
